# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from sqlalchemy import Boolean, Column, DateTime, ForeignKey
from sqlalchemy import Integer, MetaData, String, Table
from vsm.db.sqlalchemy import models

def upgrade(migrate_engine):
    # Upgrade operations go here. Don't create your own engine;
    # bind migrate_engine to your metadata
    meta = MetaData()
    meta.bind = migrate_engine

    init_nodes = Table(
        'init_nodes', meta,
        Column('id', Integer, primary_key=True, nullable=False),
        Column('host', String(length=255), nullable=True),
        Column('service_id', Integer(), nullable=True),
        Column('raw_ip', String(length=255), nullable=True),
        Column('primary_public_ip', String(length=255), nullable=True),
        Column('secondary_public_ip', String(length=255), nullable=True),
        Column('cluster_ip', String(length=255), nullable=True),
        Column('zone_id', Integer(), nullable=True),
        Column('mds', String(length=255), nullable=True),
        Column('type', String(length=255), nullable=True),
        Column('cluster_id', Integer, ForeignKey(models.Cluster.id), nullable=True),
        Column('data_drives_number', String(length=255), nullable=True),
        Column('status', String(length=255), nullable=True),
        Column('pre_status', String(length=255), nullable=True),
        Column('id_rsa_pub', String(length=1024), nullable=True),
        Column('created_at', DateTime(timezone=False)),
        Column('updated_at', DateTime(timezone=False)),
        Column('deleted_at', DateTime(timezone=False)),
        Column('deleted', Boolean(create_constraint=True, name=None)),
    )

    try:
        init_nodes.create()
    except Exception:
        meta.drop_all(tables=[init_nodes])
        raise

def downgrade(migrate_engine):
    meta = MetaData()
    meta.bind = migrate_engine

    init_nodes = Table('init_nodes',
                        meta,
                        autoload=True)
    init_nodes.drop()
