# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# Common Data
# Please input your common data here.

# ceph version and code
CEPH_FIREFLY = "firefly"
CEPH_GIANT = "giant"
CEPH_HAMMER = "hammer"
CEPH_INFERNALIS = "infernalis"
CEPH_JEWEL = "jewel"
CEPH_VERSION_CODE = [("0.80", CEPH_FIREFLY),
                     ("0.87", CEPH_GIANT),
                     ("0.94", CEPH_HAMMER),
                     ("9.2", CEPH_INFERNALIS),
                     ("10.2", CEPH_JEWEL)]
